package es.upm.dit.adsw.tema2.balls.e_exchange.a_threads;

import java.util.Random;

public class MyExchanger<V> {
    private V v1;   // from thread 1
    private V v2;   // from thread 2

    private int state = 0;
    // 0 - init or cycle finished
    // 1 - thread 1 arrived - waiting for thread 2
    // 2 - thread 2 done - waiting to restart thread 1

    public synchronized V exchange(V x)
            throws InterruptedException {
        while (state == 2)
            wait();
        if (state == 0) {
            v1 = x;
            state = 1;
            while (state == 1)
                wait();
            state = 0;
            notifyAll();
            return v2;
        } else {
            v2 = x;
            state = 2;
            notifyAll();
            return v1;
        }
    }

    // smoke test
    public static void main(String[] args) {
        MyExchanger<String> exchanger = new MyExchanger<>();
        Robotx r1 = new Robotx("A", 1, exchanger);
        Robotx r2 = new Robotx("B", 1, exchanger);
        r1.start();
        r2.start();
    }

    private static class Robotx extends Thread {
        private Random random = new Random();
        private final String id;
        private int cnt;
        private MyExchanger<String> exchanger;

        public Robotx(String id, int cnt, MyExchanger<String> exchanger) {
            this.id = id;
            this.cnt = cnt;
            this.exchanger = exchanger;
        }

        @Override
        public void run() {
            while (true) {
                try {
                    int ms = 100 + random.nextInt(2000);
                    Thread.sleep(ms);
                    String received = exchanger.exchange(id + cnt++);
                    System.out.printf("%s: %s%n", id, received);
                    if (received.charAt(0) == id.charAt(0))
                        System.out.println("ERROR");
                } catch (Exception e) {
                    System.out.println("ERROR");
                }
            }
        }
    }
}
