package es.upm.dit.adsw.tema2.balls.e_exchange.a_threads;

import javax.swing.*;
import java.awt.*;
import java.util.ArrayList;

/**
 * Dibujos en pantalla.
 *
 * @author jam
 * @version 26.1.2015
 */
public class Screen {
    private final JFrame frame;
    private final JPanel panel;
    private final ArrayList<Thing> thingList = new ArrayList<Thing>();
    private int cx;
    private int cy;
    private int r = 100;

    public Screen(String title, int width, int height) {
        frame = new JFrame(title);
        frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);

        cx = width / 2;
        cy = height / 2;

        panel = new MyJPanel();
        panel.setPreferredSize(new Dimension(width, height));
        frame.getContentPane().add(panel, BorderLayout.CENTER);
        frame.pack();
        frame.setVisible(true);
    }

    public boolean inGreen(int x, int y) {
        int dx = x - cx;
        int dy = y - cy;
        return Math.sqrt(dx * dx + dy * dy) < r;
    }

    public void add(Thing thing) {
        synchronized (thingList) {
            thingList.add(thing);
        }
    }

    public void remove(Thing thing) {
        synchronized (thingList) {
            thingList.remove(thing);
        }
    }

    public void paint() {
        frame.repaint();
    }

    public int getWidth() {
        return panel.getWidth();
    }

    public int getHeight() {
        return panel.getHeight();
    }

    public interface Thing {
        void paint(Graphics2D g);
    }

    private class MyJPanel
            extends JPanel {
        @Override
        public void paint(Graphics g) {
            int width = panel.getWidth();
            int height = panel.getHeight();

            g.setColor(Color.WHITE);
            g.fillRect(0, 0, width, height);

            g.setColor(Color.GREEN);
            g.fillOval(cx - r, cy - r, 2 * r, 2 * r);

            synchronized (thingList) {
                for (Thing thing : thingList)
                    thing.paint((Graphics2D) g);
            }
        }
    }
}
