package es.upm.dit.adsw.tema2.contador;

import java.util.concurrent.Semaphore;

public class Meter {
    private static final int TIMES = 10000;

    private static int[] N_THREADS = {1, 2, 4, 8, 16, 32};
    private static int[] ALGOS = {2, 3, 4, 5, 7};
    private static long[][] data = new long[N_THREADS.length][ALGOS.length];

    public static void main(String[] args) {
        for (int a : ALGOS) {
            for (int nt : N_THREADS) {
                long dt = test(nt, a);
                System.out.printf("%d;%d;%d%n", a, nt, dt);
            }
        }
    }

    private static long test(int nTh, int algo) {
        switch (algo) {
            case 2:
                return test2(nTh);
            case 3:
                return test3(nTh);
            case 4:
                return test4(nTh);
            case 5:
                return test5(nTh);
            case 7:
                return test7(nTh);
        }
        return 0;
    }

    private static long test2(int nTh) {
        final Contador2 cnt = new Contador2();
        for (int i = 0; i < TIMES; i++)
            cnt.incrementa(1);
        long t0 = System.nanoTime();
        for (int id = 0; id < nTh; id++) {
            Thread th = new Thread() {
                public void run() {
                    for (int i = 0; i < 4 * TIMES; i++)
                        cnt.incrementa(1);
                }
            };
            th.start();
        }
        long t2 = System.nanoTime();
        return (t2 - t0) / nTh;
    }

    private static long test3(int nTh) {
        return 0;
    }

    private static long test4(int nTh) {
        final Semaphore start = new Semaphore(0);
        final Semaphore stop = new Semaphore(0);
        final Contador4 cnt = new Contador4();
        for (int i = 0; i < TIMES; i++)
            cnt.incrementa(1);
        for (int id = 0; id < nTh; id++) {
            Thread th = new Thread() {
                public void run() {
                    try {
                        start.acquire();
                        for (int i = 0; i < 4 * TIMES; i++)
                            cnt.incrementa(1);
                        stop.release();
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }
            };
            th.start();
        }
        long t0 = System.nanoTime();
        start.release(nTh);
        try {
            stop.acquire(nTh);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        long t2 = System.nanoTime();
        return (t2 - t0) / nTh;
    }

    private static long test5(int nTh) {
        return 0;
    }

    private static long test7(int nTh) {
        return 0;
    }
}
